/*! @file HAL_GPIO.c
 * @brief This file implement the HAL for the GPIO peripheral.
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████


*/

#include <stdint.h>
#include "stdlib.h"
#include "string.h"
#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include "HAL_GPIO.h"

//! Maximum number of GPIO pins on NRF53
#define MAXIMUM_NUM_OF_PINS      32

//! Structure registering all the GPIO devicetree configurations
// static struct gpio_dt_spec gpioSpecification[MAXIMUM_NUM_OF_PINS]; 
//! Structure registering the GPIO devicetree extra flags
// uint32_t extraFlags [MAXIMUM_NUM_OF_PINS];
//! Structure to store the callback data
static struct gpio_callback pinCallbackData;

/*! This function set the configuration for a set of pins.
 * @param[in] pPinDescriptorArray Pointer to the array of pin descriptors   \
 * to be configured
 * @param[in] nPins Number of pins in the array pointed by nPinDescriptor
 */
const int ABTC_HAL_GPIO_Config( const void *pPinDescriptorArray,        \
                                    uint8_t nPins )
{
    tGPIOPinDescriptor *pPinConfig;
    struct gpio_dt_spec gpioSpecification;
    uint32_t extraFlags = 0;
    uint8_t index;
    uint8_t configureInterruptFlag = 0;
    
    if ( NULL == pPinDescriptorArray )
    {
        return HAL_GPIO_ERROR;
    }

    for( index = 0; index < nPins; index++ )
    {
        memset(&gpioSpecification, 0, sizeof(struct gpio_dt_spec));
        extraFlags = 0;
        configureInterruptFlag = 0;

        pPinConfig = (tGPIOPinDescriptor *)((char *)pPinDescriptorArray +  \
                    (index * sizeof(tGPIOPinDescriptor))) ;

        gpioSpecification.pin = pPinConfig ->pinIdentifier;

        switch (pPinConfig->portConfig)
        {
            case HAL_GPIO_PORT_A:
                gpioSpecification.port =                             \
                        DEVICE_DT_GET(DT_NODELABEL(gpio0));
                break;
            case HAL_GPIO_PORT_B:
                gpioSpecification.port =                             \
                        DEVICE_DT_GET(DT_NODELABEL(gpio1));
                break;
            default:
                gpioSpecification.port =                             \
                        DEVICE_DT_GET(DT_NODELABEL(gpio0));
                break;
        }

        switch (pPinConfig->pinInputOutputConfig)
        {
            case HAL_GPIO_MODE_DISABLE:
                gpioSpecification.dt_flags &= ~GPIO_DISCONNECTED;
                break;
            case HAL_GPIO_MODE_INPUT:
                extraFlags |= GPIO_INPUT;
                break;
            case HAL_GPIO_MODE_OUTPUT_PP:
            case HAL_GPIO_MODE_AF_PP:
                extraFlags |= GPIO_OUTPUT_INACTIVE;
                gpioSpecification.dt_flags |= GPIO_ACTIVE_HIGH;
                gpioSpecification.dt_flags &= ~GPIO_PUSH_PULL;
                break;
            case HAL_GPIO_MODE_OUTPUT_OD:
            case HAL_GPIO_MODE_AF_OD:
                extraFlags |= GPIO_OUTPUT_ACTIVE;
                gpioSpecification.dt_flags |= GPIO_ACTIVE_LOW;
                gpioSpecification.dt_flags |= GPIO_SINGLE_ENDED;
                gpioSpecification.dt_flags |= GPIO_LINE_OPEN_DRAIN;
                break;
            case HAL_GPIO_MODE_ANALOG:
                extraFlags |= GPIO_OUTPUT_ACTIVE;
                gpioSpecification.dt_flags |= GPIO_ACTIVE_LOW;
                break;
            case HAL_GPIO_MODE_IT_RISING:
            case HAL_GPIO_MODE_EVT_RISING:
                extraFlags |= GPIO_INPUT;
                extraFlags |= GPIO_INT_EDGE_RISING;
                configureInterruptFlag = 1;
                break;
            case HAL_GPIO_MODE_IT_FALLING:
            case HAL_GPIO_MODE_EVT_FALLING:
                extraFlags |= GPIO_INPUT;
                extraFlags |= GPIO_INT_EDGE_FALLING;
                configureInterruptFlag = 1;
                break;
            case HAL_GPIO_MODE_IT_RISING_FALLING:
            case HAL_GPIO_MODE_EVT_RISING_FALLING:
                extraFlags |= GPIO_INPUT;
                extraFlags |= GPIO_INT_EDGE_BOTH;
                configureInterruptFlag = 1;
                break;
            default:
                gpioSpecification.dt_flags &= ~GPIO_DISCONNECTED;
                extraFlags &= ~GPIO_DISCONNECTED;
                break;
        }

        switch (pPinConfig->pullConfig)
        {
            case HAL_GPIO_NOPULL:
                break;
            case HAL_GPIO_PULL_UP:
                gpioSpecification.dt_flags |= GPIO_PULL_UP;
                break;
            case HAL_GPIO_PULL_DOWN:
                gpioSpecification.dt_flags |= GPIO_PULL_DOWN;
                break;
            default:
                break;
        };

        gpio_pin_configure(                                                 \
                            gpioSpecification.port,                         \
                            gpioSpecification.pin,                          \
                            gpioSpecification.dt_flags | extraFlags);

        if( 1 == configureInterruptFlag )
        {
            gpio_pin_interrupt_configure(                                   \
                                gpioSpecification.port,                     \
                                gpioSpecification.pin,                      \
                                gpioSpecification.dt_flags | extraFlags);
        }                    
        if( ( HAL_GPIO_MODE_OUTPUT_PP == pPinConfig->pinInputOutputConfig ) ||\
            ( HAL_GPIO_MODE_OUTPUT_OD == pPinConfig->pinInputOutputConfig ) ){
                ABTC_HAL_GPIO_Set_Output( pPinConfig, pPinConfig->defaultValue );
            }
        
    }
    return HAL_GPIO_SUCCESS;
}

/*! This function sets a digital output to 0 or 1
 * @param[in] pPinDescriptor Descriptor of the pin
 * @param[in] value Only LSB will be used to set the output to 0 or 1
 */
int ABTC_HAL_GPIO_Set_Output( const void *pPinDescriptor,                   \
                                    uint8_t value )
{
    tGPIOPinDescriptor *pPinConfig;
    struct gpio_dt_spec gpioSpecification;

    if ( NULL == pPinDescriptor )
    {
        return HAL_GPIO_ERROR;
    }

    pPinConfig = (tGPIOPinDescriptor *)((char *)pPinDescriptor);

    switch (pPinConfig->portConfig)
    {
        case HAL_GPIO_PORT_A:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio0));
            break;
        case HAL_GPIO_PORT_B:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio1));
            break;
        default:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio0));
            break;
    }

    gpioSpecification.pin = pPinConfig ->pinIdentifier;

    gpio_pin_set(gpioSpecification.port, gpioSpecification.pin, value);

    return HAL_GPIO_SUCCESS;
}

/*! This function reads a digital input value
 * @param[in] pPinDescriptor Descriptor of the pin
 * @return Read value
 */
uint8_t ABTC_HAL_GPIO_Read_Input(const void *pPinDescriptor)
{
    uint8_t readInput;
    tGPIOPinDescriptor *pPinConfig;
    struct gpio_dt_spec gpioSpecification;

    if ( NULL == pPinDescriptor )
    {
        return HAL_GPIO_ERROR;
    }

    pPinConfig = (tGPIOPinDescriptor *)((char *)pPinDescriptor);

    switch (pPinConfig->portConfig)
    {
        case HAL_GPIO_PORT_A:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio0));
            break;
        case HAL_GPIO_PORT_B:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio1));
            break;
        default:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio0));
            break;
    }

    gpioSpecification.pin = pPinConfig ->pinIdentifier;

    readInput = gpio_pin_get(gpioSpecification.port, gpioSpecification.pin);

    return readInput;
}

/*! This function sets the interrupt of a gpio pin
 *  @param[in] pPinDescriptor Descriptor of the pin
 *  @param[in] pInterruptFunction Pointer to the interrupt function
 */
uint8_t ABTC_HAL_GPIO_Set_Interrupt( const void *pPinDescriptor, 			\
		 	 	 	 	 	 	   void * pInterruptFunction)
{
    tGPIOPinDescriptor *pPinConfig;
    struct gpio_dt_spec gpioSpecification;
    gpio_callback_handler_t callbackHandler;

    if ( NULL == pPinDescriptor || NULL == pInterruptFunction)
    {
        return HAL_GPIO_ERROR;
    }

    pPinConfig = (tGPIOPinDescriptor *)((char *)pPinDescriptor);

    switch (pPinConfig->portConfig)
    {
        case HAL_GPIO_PORT_A:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio0));
            break;
        case HAL_GPIO_PORT_B:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio1));
            break;
        default:
            gpioSpecification.port = DEVICE_DT_GET(DT_NODELABEL(gpio0));
            break;
    }

    gpioSpecification.pin = pPinConfig ->pinIdentifier;

    callbackHandler = pInterruptFunction;

    gpio_init_callback(&pinCallbackData, callbackHandler,    \
                    BIT(gpioSpecification.pin));
    gpio_add_callback(gpioSpecification.port,            \
                    &pinCallbackData);

    return HAL_GPIO_SUCCESS;
}