/*! @file HAL_GPIO.h
 * @brief This file implement the HAL for the GPIO peripherals.
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/

#ifndef HAL_GPIO_H_
#define HAL_GPIO_H_

#include <stdint.h>

//! Possible input - output values of the GPIO pins
#define HAL_GPIO_PIN_LOW        0
#define HAL_GPIO_PIN_HIGH       1

/*!@enum eGPIOErrorType
 * @brief Enumerator for the type of GPIO error functions return
 * @typedef tGPIOErrorType
 * @brief Data type related to the enum eGPIOErrorType
 */
typedef enum eGPIOErrorType{
    //! GPIO error
    HAL_GPIO_ERROR = -1,
    //! GPIO success
    HAL_GPIO_SUCCESS = 1,
    //! Number of error types supported
    HAL_GPIO_MAX_NUMBER_OF_ERROR_TYPES
} tGPIOErrorType;

/*! @enum eGPIOPortConfig
 *  @brief Enumeration with the different port configuration
 *  @typedef tGPIOPortConfig
 *  @brief Data type related to the enum eGPIOPortConfig
 */
typedef enum eGPIOPortConfig{     
    //! Port A
    HAL_GPIO_PORT_A,
    //! Port B
    HAL_GPIO_PORT_B,
    //! Port C
    HAL_GPIO_PORT_C,
    //! Port D
    HAL_GPIO_PORT_D,
    //! Port E
    HAL_GPIO_PORT_E,
    //! Port F
    HAL_GPIO_PORT_F,
    //! Port G
    HAL_GPIO_PORT_G,
    //! Port H
    HAL_GPIO_PORT_H,
    //! Max number of ports
    HAL_GPIO_PIN_PORT_MAX_NUMBER,
} tGPIOPortConfig;

/*! @enum eGPIOPinInputOutputConfig
 *  @brief Enumerator for the Input / Output Configuration
 *  @typedef tGPIOPinInputOutputConfig
 *  @brief Data type related to the enum eGPIOPinInputOutputConfig
 */
typedef enum eGPIOPinInputOutputConfig{
    //! Pin disable input and output
    HAL_GPIO_MODE_DISABLE,
    //! Input mode
    HAL_GPIO_MODE_INPUT,
    //! Output mode push pull
    HAL_GPIO_MODE_OUTPUT_PP,
    //! Output mode open drain
    HAL_GPIO_MODE_OUTPUT_OD,
    //! Alternative function push pull
    HAL_GPIO_MODE_AF_PP,
    //! Alternative function open drain
    HAL_GPIO_MODE_AF_OD,
    //! Analog mode
    HAL_GPIO_MODE_ANALOG,
    //! External interruption mode low to high
    HAL_GPIO_MODE_IT_RISING,
    //! External interruption mode high to low
    HAL_GPIO_MODE_IT_FALLING,
    //! External interruption mode low to high and high to low
    HAL_GPIO_MODE_IT_RISING_FALLING,
    //! External event mode low to high
    HAL_GPIO_MODE_EVT_RISING,
    //! External event mode high to low
    HAL_GPIO_MODE_EVT_FALLING,
    //! External event mode low to high and high to low
    HAL_GPIO_MODE_EVT_RISING_FALLING,
    //! Max number of possible config modes
    HAL_GPIO_MODE_MAX_NUMBER,
} tGPIOPinInputOutputConfig;

/*! @enum eGPIOPullConfig
 *  @brief Enumerator with the push pull configuration
 *  @typedef tGPIOPullConfig
 *  @brief Data type related to the enum eGPIOPullConfig
 */
typedef enum eGPIOPullConfig{
    //! Pin pull up resistor disable
    HAL_GPIO_NOPULL,
    //! Pin pull up resistor enable
    HAL_GPIO_PULL_UP,
    //! Pin pull down resistor enable
    HAL_GPIO_PULL_DOWN,
    //! Max number of possible pull configs
    HAL_GPIO_PULL_MAX_NUMBER,
} tGPIOPullConfig;

/*! @enum eGPIOValues
 * @brief Enumeration of the GPIO values (0 or 1)
 * @typedef tGPIOValues
 * @brief Data type related to the enum eGPIOValues
 */
typedef enum eGPIOValues{
    //! Value 0
    HAL_GPIO_VALUE_LOW,
    //! Value 1
    HAL_GPIO_VALUE_HIGH,
    //! Number of GPIO Values
    GPIO_NUM_OF_VALUES
}tGPIOValues;

/*! @typedef tGPIOCallbackFunction
 *  @brief Data type related to a void pointer to a function
 */
typedef void *(*tGPIOCallbackFunction)(void);

/*! @struct sGPIOPinDescriptor
 *  @brief GPIO Pin descriptor used to configure the GPIO Pins
 *  @typedef tGPIOPinDescriptor
 *  @brief Data Type related to data structure sGPIOPinDescriptor
 */
typedef struct sGPIOPinDescriptor{
    //! Pin Identifier
    uint32_t pinIdentifier;
    //! Pin port
    tGPIOPortConfig portConfig;
    //! Input / Output identifier
    tGPIOPinInputOutputConfig pinInputOutputConfig;
    //! Pull up/down configuration
    tGPIOPullConfig pullConfig;
    //! Interrupt Callback. NULL if there is not any callback associated
    tGPIOCallbackFunction callbackFunction;
    //! Default value when the GPIO is an output, otherwise ignored
    tGPIOValues defaultValue;
} tGPIOPinDescriptor;

/*! This function set the configuration for a set of pins.
 * @param[in] pPinDescriptorArray Pointer to the array of pin descriptors   \
 * to be configured
 * @param[in] nPins Number of pins in the array pointed by nPinDescriptor
 */
const int ABTC_HAL_GPIO_Config( const void *pPinDescriptorArray,          \
                                  uint8_t nPins );

/*! This function sets a digital output to 0 or 1
 * @param[in] pPinDescriptor Descriptor of the pin
 * @param[in] value Only LSB will be used to set the output to 0 or 1
 */
int ABTC_HAL_GPIO_Set_Output( const void *pPinDescriptor,                   \
                               uint8_t value );

/*! This function reads a digital input value
 * @param[in] pPinDescriptor Descriptor of the pin
 * @return Read value
 */
uint8_t ABTC_HAL_GPIO_Read_Input(const void *pPinDescriptor);

/*! This function sets the interrupt of a gpio pin
 *  @param[in] pPinDescriptor Descriptor of the pin
 *  @param[in] pInterruptFunction Pointer to the interrupt function
 */
uint8_t ABTC_HAL_GPIO_Set_Interrupt( const void *pPinDescriptor, 				\
		 	 	 	 	 	 	   void * pInterruptFunction);
#endif /* HAL_GPIO_H_ */
