/*! @file HAL_SPI.c
 * @brief This file implements the HAL for the SPI peripherals.
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████

*/

#include <zephyr/drivers/spi.h>
#include <nrfx_spim.h>
#include "HAL_SPI.h"

//! Handle for spi master port 3 
static nrfx_spim_t spiMasterHandler = NRFX_SPIM_INSTANCE(3);

/*! Function used for initializing SPI port
 * @param[in] pDescriptor Pointer to the descriptor of the SPI port
 * @return 1 if sucess, 0 otherwise 
 */
const uint8_t ABTC_HAL_SPI_Init( const void *pDescriptor )
{
    tSPIDescriptor *pPinConfig;
    nrfx_err_t error;
    nrfx_spim_config_t spiMasterConfig =                                        \
            NRFX_SPIM_DEFAULT_CONFIG(0xFF, 0xFF, 0xFF, 0xFF);

    if ( NULL == pDescriptor )
    {
        return HAL_SPI_INIT_ERROR;
    }

	pPinConfig = (tSPIDescriptor *)(pDescriptor);

    spiMasterConfig.sck_pin = NRF_GPIO_PIN_MAP(                             \
                            pPinConfig->sclkPortConfig,                     \
                            pPinConfig->sclkPinIdentifier);
    spiMasterConfig.mosi_pin = NRF_GPIO_PIN_MAP(                            \
                            pPinConfig->mosiPortConfig,                     \
                            pPinConfig->mosiPinIdentifier);
    spiMasterConfig.miso_pin = NRF_GPIO_PIN_MAP(                            \
                            pPinConfig->misoPortConfig,                     \
                            pPinConfig->misoPinIdentifier);
    spiMasterConfig.ss_pin = NRF_GPIO_PIN_MAP(                              \
                            pPinConfig->ccsPortConfig,                      \
                            pPinConfig->ccsPinIdentifier);

    switch(pPinConfig->spiFrequency)
    {
        case HAL_SPI_FREQUENCY_125K:
            spiMasterConfig.frequency = 125000;// NRF_SPIM_FREQ_125K;
            break;
        case HAL_SPI_FREQUENCY_250K:
            spiMasterConfig.frequency = 250000;//NRF_SPIM_FREQ_250K;
            break;
        case HAL_SPI_FREQUENCY_500K:
            spiMasterConfig.frequency = 500000;//NRF_SPIM_FREQ_500K;
            break;
        case HAL_SPI_FREQUENCY_1M:
            spiMasterConfig.frequency = 1000000;//NRF_SPIM_FREQ_1M;
            break;
        case HAL_SPI_FREQUENCY_2M:
            spiMasterConfig.frequency = 2000000;//NRF_SPIM_FREQ_2M;
            break;
        case HAL_SPI_FREQUENCY_4M:
            spiMasterConfig.frequency = 4000000;//NRF_SPIM_FREQ_4M;
            break;
        default:
            spiMasterConfig.frequency = 1000000;//NRF_SPIM_FREQ_1M;
            break;
    }

    switch(pPinConfig->spiClockPolarity)
    {
        case HAL_SPI_CPOL_LOW:
            switch(pPinConfig->spiClockPhase)
            {
                case HAL_SPI_CPHA_LOTOHI:
                    spiMasterConfig.mode = NRF_SPIM_MODE_0;
                    break;
                case HAL_SPI_CPHA_HITOLO:
                    spiMasterConfig.mode = NRF_SPIM_MODE_1;
                    break;
                default:
                    break;
            }
            break;
        case HAL_SPI_CPOL_HIGH:
            switch(pPinConfig->spiClockPhase)
            {
                case HAL_SPI_CPHA_LOTOHI:
                    spiMasterConfig.mode = NRF_SPIM_MODE_2;
                    break;
                case HAL_SPI_CPHA_HITOLO:
                    spiMasterConfig.mode = NRF_SPIM_MODE_3;
                    break;
                default:
                    break;
            }
            break;
        default:
            spiMasterConfig.mode = NRF_SPIM_MODE_0;
            break;
    }

	spiMasterConfig.skip_gpio_cfg = false;
	spiMasterConfig.skip_psel_cfg = false;

    error = nrfx_spim_init(&spiMasterHandler, &spiMasterConfig, NULL, NULL);

	if (error != NRFX_SUCCESS) 
    {
		return HAL_SPI_INIT_ERROR;
	}

    return HAL_SPI_INIT_OK;
}

/*! Function for sending a frame on the SPI port. This function is not blocking.
 * So it should be consulted first if the SPI bus is ready or not.
 * @param[in] pDescriptor SPI Handler of the port where the bytes will be sent
 * @param[in] pDataWrite Pointer to the array of bytes to be sent
 * @param[in] pDataRead Pointer to the array where the bytes will be received
 * @param[in] nBytes Number of bytes in the buffer pointed by any pBytes pointer
 * @return Number of bytes sent
 */
uint32_t ABTC_HAL_SPI_Transfer( const void *pDescriptor, 										\
                                const uint8_t *pDataWrite, 										\
                                uint8_t *pDataRead, 											\
                                uint32_t nBytes )
{
    nrfx_err_t error;

    nrfx_spim_xfer_desc_t transferDescriptor = NRFX_SPIM_XFER_TRX(pDataWrite, nBytes, pDataRead, nBytes);

	error = nrfx_spim_xfer(&spiMasterHandler, &transferDescriptor, 0);

	if (error != NRFX_SUCCESS) 
    {		
		return 	HAL_SPI_TRANSACTION_ERROR;
	}
    return HAL_SPI_TRANSACTION_OK;
}

/*! This function uninits the SPI
 *
 */
void ABTC_HAL_SPI_Uninit( void )
{
    nrfx_spim_uninit(&spiMasterHandler);
}