/*! @file HAL_SPI.h
 * @brief This file implement the HAL for the SPI peripherals.
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/

#ifndef __HAL_SPI_H__
#define __HAL_SPI_H__

#include <stdint.h>

/*!@enum eSPIError
 * @brief Enumeration with the types of errors in SPI Module
 * @typedef tSPIError
 * @brief Data type associated to the enum eSPIError
 */
typedef enum eSPIError {
    //! SPI Init Ok
    HAL_SPI_INIT_OK,
    //! SPI Init Error
	HAL_SPI_INIT_ERROR,
	//! SPI Transanction Ok
	HAL_SPI_TRANSACTION_OK,
	//! SPI Transanction Error
	HAL_SPI_TRANSACTION_ERROR,
	//! SPI Num of errors
    HAL_SPI_NUM_OF_ERRORS
} tSPIError;

/*!@enum eSPITransferSize
 * @brief Enumerator for the data transfer size configuration
 * @typedef tSPITransferSize
 * @brief Data type related to the enum eSPITransferSize
 */
typedef enum eSPITransferSize{
	//! 4 bits data transfer size
	HAL_SPI_DATA_SIZE_4BIT = 0,
	//! 5 bits data transfer size
	HAL_SPI_DATA_SIZE_5BIT,
	//! 6 bits data transfer size
	HAL_SPI_DATA_SIZE_6BIT,
	//! 7 bits data transfer size
	HAL_SPI_DATA_SIZE_7BIT,
	//! 8 bits data transfer size
	HAL_SPI_DATA_SIZE_8BIT,
	//! 10 bits data transfer size
	HAL_SPI_DATA_SIZE_10BIT,
	//! 16 bits data transfer size
	HAL_SPI_DATA_SIZE_16BIT,
	//! Max Number of SPI modes
	HAL_SPI_MAX_OF_DATA_SIZE
}tSPITransferSize;

/*!@enum eSPIFrequency
 * @brief Enumerator for the frequency and clock prescaler configuration
 * @typedef tSPIFrequency
 * @brief Data type related to the enum eSPIFrequency
*/
typedef enum eSPIFrequency{
	//! Baudrate is 125 kBits/s -- Prescaler is 256
	HAL_SPI_FREQUENCY_125K,
	//! Baudrate is 250 kBits/s -- Prescaler is 128
	HAL_SPI_FREQUENCY_250K,
	//! Baudrate is 500 kBits/s -- Prescaler is 64
	HAL_SPI_FREQUENCY_500K,
	//! Baudrate is 1000 kBits/s -- Prescaler is 32
	HAL_SPI_FREQUENCY_1M,
	//! Baudrate is 2 MBits/s -- Prescaler is 16
	HAL_SPI_FREQUENCY_2M,
	//! Baudrate is 4 MBits/s -- Prescaler is 8
	HAL_SPI_FREQUENCY_4M,
	//! Baudrate is 8 MBits/s -- Prescaler is 4
	HAL_SPI_FREQUENCY_8M,
	//! Baudrate is 16 MBits/s -- Prescaler is 2
	HAL_SPI_FREQUENCY_16M,
	//! Max Number of SPI modes
	HAL_SPI_MAX_OF_FREQUENCY
}tSPIFrequency;

/*!@enum eSPIClockPolarity
 * @brief Enumerator for the clock polarity configuration
 * @typedef tSPIClockPolarity
 * @brief Data type related to the enum eSPIClockPolarity
 */
typedef enum eSPIClockPolarity{
	//! SPI low clock polarity
	HAL_SPI_CPOL_LOW,
	//! SPI high clock polarity
	HAL_SPI_CPOL_HIGH,
	//! Max Number of Clock Polarity configurations
	HAL_SPI_MAX_OF_CPOL
}tSPIClockPolarity;

/*!@enum eSPIClockPhase
 * @brief Enumerator for the clock phase configuration
 * @typedef tSPIClockPhase
 * @brief Data type related to the enum eSPIClockPhase
 */
typedef enum eSPIClockPhase{
	//! Low to High Phase
	HAL_SPI_CPHA_LOTOHI,
	//! High to Low Phase
	HAL_SPI_CPHA_HITOLO,
	//! Max Number of clock phase configurations
	HAL_SPI_MAX_OF_CPHA
}tSPIClockPhase;

/*! @enum eSPIPortConfig
 *  @brief Enumeration with the different port configuration
 *  @typedef tSPIPortConfig
 *  @brief Data type related to the enum eSPIPortConfig
 */
typedef enum eSPIPortConfig{
	//! Port A
	HAL_SPI_PORT_A,
	//! Port B
	HAL_SPI_PORT_B,
	//! Port C
	HAL_SPI_PORT_C,
	//! Port D
	HAL_SPI_PORT_D,
	//! Port E
	HAL_SPI_PORT_E,
	//! Port F
	HAL_SPI_PORT_F,
	//! Port G
	HAL_SPI_PORT_G,
	//! Port H
	HAL_SPI_PORT_H,
	//! Max Number of GPIO Port Configurations
	HAL_SPI_PORT_MAX,
} tSPIPortConfig;

/*! @struct sSPIDescriptor
 *  @brief Descriptor for the SPI Configuration
 *  @typedef tSPIDescriptor
 *  @brief Data Type related to the struct sSPIDescriptor
 */
typedef struct sSPIDescriptor{
	//! MISO Pin Identifier
	uint32_t misoPinIdentifier;
	//! MISO Port Identifier
	tSPIPortConfig misoPortConfig;
	//! MOSI Pin Identifier
	uint32_t mosiPinIdentifier;
	//! MOSI Port Identifier
	tSPIPortConfig mosiPortConfig;
	//! SCLK Pin Identifier
	uint32_t sclkPinIdentifier;
	//! SCLK Port Identifier
	tSPIPortConfig sclkPortConfig;
	//! CCS Pin Identifier
	uint32_t ccsPinIdentifier;
	//! CCS Port Identifier
	tSPIPortConfig ccsPortConfig;
	//! Data size to transfer
	tSPITransferSize spiTransferSize;
	//! Frequency
	tSPIFrequency spiFrequency;
	//! Clock Polarity (CPOL)
	tSPIClockPolarity spiClockPolarity;
	//! Clock Phase (CPHA)
	tSPIClockPhase spiClockPhase;
}tSPIDescriptor;

/*! Function used for initializing SPI port
 * @param[in] pDescriptor Pointer to the descriptor of the SPI port
 * @return 1 if sucess, 0 otherwise 
 */
const uint8_t ABTC_HAL_SPI_Init( const void *pDescriptor );

/*! Function for sending a frame on the SPI port. This function is not blocking.
 * So it should be consulted first if the SPI bus is ready or not.
 * @param[in] pDescriptor SPI Handler of the port where the bytes will be sent
 * @param[in] pDataWrite Pointer to the array of bytes to be sent
 * @param[in] pDataRead Pointer to the array where the bytes will be received
 * @param[in] nBytes Number of bytes in the buffer pointed by any pBytes pointer
 * @return Number of bytes sent
 */
uint32_t ABTC_HAL_SPI_Transfer( const void *pDescriptor, 										\
                                const uint8_t *pDataWrite, 										\
                                uint8_t *pDataRead, 											\
                                uint32_t nBytes );

/*! This function uninits the SPI
 *
 */
void ABTC_HAL_SPI_Uninit( void );

#endif /* __HAL_SPI_H__ */
