/*! @file MX25R6435F.c
 * @brief This file implements the functions for communicating with
 * MX25R6435F external flash memory
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/
 
#include <stdio.h>
#include <zephyr/kernel.h>
#include "HAL_SPI.h"
#include "HAL_GPIO.h"
#include "MX25R6435F.h"
#include "Hardware_Init.h"

//! Size of buffer
#define MX25R6435F_BUFFER_SIZE                                              261

//! Pointer od the SPI Descriptor
static void *pSpiMX25R6435F = NULL;

//! Data buffer TX
static uint8_t bufferTX[ MX25R6435F_BUFFER_SIZE ];
//! Data buffer RX
static uint8_t bufferRX[ MX25R6435F_BUFFER_SIZE ];

static void MX25R6435F_Write_Enable( void );
static uint16_t MX25R6435F_Read_Status_Register( void );
static void MX25R6435F_Wait_For_WIP( void );
static void MX25R6435F_Wait_For_WEL( void );

/*! Function for initializing MX25R6435F
 */
void MX25R6435F_Init( void )
{

    Hardware_Init_Get_Descriptor( &pSpiMX25R6435F, 
                                  HARDWARE_INIT_SPI_DESCRIPTOR);

}

/*! Function for erase whole chiip (CE) on MX25R6435F
 */
void MX25R6435F_Chip_Erase( void )
{
  if( NULL == pSpiMX25R6435F ){
    MX25R6435F_Init();
    /*
      if( NULL == pSpiMX25R6435F ){
        // Still null, execute some error routine if any
      }
    */
  }
	
  MX25R6435F_Write_Enable();
  MX25R6435F_Wait_For_WEL();

  bufferTX[0] =  MX25R6435F_COMMAND_CHIP_ERASE ;
  ABTC_HAL_SPI_Transfer(pSpiMX25R6435F, bufferTX, bufferRX, 1);

  MX25R6435F_Wait_For_WIP();

}

/*! Function for erase the selected block (BE) on MX25R6435F
* @param[in] address Address of the block to be erased
 */
void MX25R6435F_Block_Erase( uint32_t address )
{

	uint8_t bufferIndex = 0;

  if( NULL == pSpiMX25R6435F ){
    MX25R6435F_Init();
    /*
      if( NULL == pSpiMX25R6435F ){
        // Still null, execute some error routine if any
      }
    */
  }
  MX25R6435F_Write_Enable();
  MX25R6435F_Wait_For_WEL();

  bufferTX[ bufferIndex++ ] = MX25R6435F_COMMAND_BLOCK_ERASE;
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 16);
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 8);
  bufferTX[ bufferIndex++ ] = (uint8_t)address;

  ABTC_HAL_SPI_Transfer(pSpiMX25R6435F, bufferTX, bufferRX, bufferIndex);

  MX25R6435F_Wait_For_WIP();

}


/*! Function for erase the selected sector (SE) on MX25R6435F
* @param[in] address Address of the sector to be erased
 */
void MX25R6435F_Sector_Erase( uint32_t address )
{
  uint8_t bufferIndex = 0;

  MX25R6435F_Write_Enable();
  MX25R6435F_Wait_For_WEL();

  bufferTX[ bufferIndex++ ] = MX25R6435F_COMMAND_SECTOR_ERASE;
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 16);
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 8);
  bufferTX[ bufferIndex++ ] = (uint8_t)address;

  ABTC_HAL_SPI_Transfer(pSpiMX25R6435F, bufferTX, bufferRX, bufferIndex);

  MX25R6435F_Wait_For_WIP();

}

/*! Function for program the selected page (PP) on MX25R6435F
* @param[in] address Address of the page to be programed
* @param[in] pData Pointer to the array of bytes to be programed
* @param[in] nData Number of bytes to be programed
 */
void MX25R6435F_Page_Program( uint32_t address, uint8_t *pData, uint32_t nData )
{
  uint16_t bufferIndex = 0;

  MX25R6435F_Write_Enable();
  MX25R6435F_Wait_For_WEL();


  bufferTX[ bufferIndex++ ] = MX25R6435F_COMMAND_PAGE_PROGRAM;
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 24);
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 16);
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 8);
  memcpy( bufferTX + 4, pData, nData );  
  
  ABTC_HAL_SPI_Transfer(pSpiMX25R6435F, bufferTX, bufferRX, nData + bufferIndex);
  
  MX25R6435F_Wait_For_WIP();
}

/*! Function for read n bytes (READ) on MX25R6435F
* @param[in] address Address of the page to be readed
* @param[in] pData Pointer to the array of bytes to be saved the data readed
* @param[in] nData Number of bytes to be readed
 */
void MX25R6435F_Page_Read( uint32_t address, uint8_t *pData, uint32_t nData )
{
  uint16_t bufferIndex = 0;

  if( NULL == pData ){
    return;
  }
  bufferTX[ bufferIndex++ ] = MX25R6435F_COMMAND_READ_DATA;
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 24);
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 16);
  bufferTX[ bufferIndex++ ] = (uint8_t)(address >> 8);
  
  ABTC_HAL_SPI_Transfer(pSpiMX25R6435F, bufferTX, bufferRX, nData + bufferIndex );

  memcpy( pData, bufferRX + bufferIndex, nData);
}

/*! Function for enabling write (WREN) on MX25R6435F
 */
static void MX25R6435F_Write_Enable( void )
{

	bufferTX[0] = MX25R6435F_COMMAND_WRITE_ENABLE;

  ABTC_HAL_SPI_Transfer(pSpiMX25R6435F, bufferTX, bufferRX, 1);

}

/*! Function for read status register (RDSR) on MX25R6435F
 */
static uint16_t MX25R6435F_Read_Status_Register( void )
{
  uint16_t statusRegister;

	bufferTX[0] = MX25R6435F_COMMAND_READ_STATUS_REGISTER;
  ABTC_HAL_SPI_Transfer(pSpiMX25R6435F, bufferTX, bufferRX, 2);

  statusRegister =  bufferRX[ 1 ];
  return statusRegister;
}

/*! Function for check write in progress that indicates the device is busy
 */
static void MX25R6435F_Wait_For_WIP( void )
{
  while( MX25R6435F_Read_Status_Register() &                                \
                    MX25R6435F_SR_WRITE_IN_PROGRESS_BIT ){
    k_msleep(5);
  }
}


/*! Function for checking that the Write Enable Latch is Active
 */
static void MX25R6435F_Wait_For_WEL( void )
{
  while( !( MX25R6435F_Read_Status_Register() &                                \
                    MX25R6435F_SR_WRITE_ENABLE_LATCH_BIT ) ){
    k_msleep(5);
  }
}

/*! Function for read the ID from IC MX25R6435F
 * @return The ID data of the device
 */
uint32_t MX25R6435F_Read_ID(void)
{
  uint32_t readValue = 0;

  memset(bufferRX, 0, MX25R6435F_BUFFER_SIZE);

  bufferTX[0] = MX25R6435F_COMMAND_READ_ID_REGISTER;
  bufferTX[1] = 0xAA;
  bufferTX[2] = 0xAA;
  bufferTX[3] = 0xAA;

  ABTC_HAL_SPI_Transfer(pSpiMX25R6435F, bufferTX, bufferRX, 4);
 
  readValue += (uint32_t)(bufferRX[1] << 16 );
  readValue += (uint32_t)(bufferRX[2] << 8 );
  readValue += (uint32_t)(bufferRX[3]);
  
  k_msleep(100);

  return readValue;
}

/*! Function for test 
* @return 1 when success, 0 when error
 */
uint8_t MX25R6435F_Test( void )
{
  uint8_t testBufferWrite[14] = "Hello World!\r";
  uint8_t testBufferRead[14];

  if( NULL == pSpiMX25R6435F ){
    MX25R6435F_Init();
    /*
      if( NULL == pSpiMX25R6435F ){
        // Still null, execute some error routine if any
      }
    */
  }

  MX25R6435F_Read_Status_Register();
  MX25R6435F_Block_Erase(0);
  MX25R6435F_Read_Status_Register();
  MX25R6435F_Page_Program( 0, testBufferWrite, 14 );

  memset( testBufferRead, 0xFF, 14 );
  MX25R6435F_Page_Read(0, testBufferRead, 14 );

  if( memcmp(testBufferRead, testBufferWrite, 14) != 0){
    return 0;
  }

  memset( testBufferWrite, 0xFF, 14 );
  MX25R6435F_Sector_Erase(0);
  MX25R6435F_Page_Read(0, testBufferRead, 14 );
  
  if( MX25R6435F_Read_ID() != 0x00C22817 ){
    return 0;
  }
  
  return 1;
}
