/*! @file MX25R6435F.h
 * @brief This file implements the functions for communicating with
 * MX25R6435F external flash memory
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/

#include <stdint.h>

#ifndef __MX25R6435F_H_
#define __MX25R6435F_H_

//! Command to enable the next write operation
#define MX25R6435F_COMMAND_WRITE_ENABLE                                     0x06      
//! Command to disable any write operation
#define MX25R6435F_COMMAND_WRITE_DISABLE                                    0x04                      
//! Command to write the status register
#define MX25R6435F_COMMAND_WRITE_STATUS_REGISTER                            0x01 
//! Command to read the identification register
#define MX25R6435F_COMMAND_READ_ID_REGISTER                                 0x9F
//! Command to read the status register
#define MX25R6435F_COMMAND_READ_STATUS_REGISTER                             0x05
//! Command to read data
#define MX25R6435F_COMMAND_READ_DATA                                        0x03
//! Command to read the device electronic manufacturer & device ID
#define MX25R6435F_COMMAND_REMS                                             0x90
//! Sector erase command
#define MX25R6435F_COMMAND_SECTOR_ERASE                                     0x20
//! Block erase command
#define MX25R6435F_COMMAND_BLOCK_ERASE                                      0x52
//! Chip erase command
#define MX25R6435F_COMMAND_CHIP_ERASE                                       0x60
//! Command to write a flash page
#define MX25R6435F_COMMAND_PAGE_PROGRAM                                     0x02

/* Status register bits */
//! Write enable latch bit
#define MX25R6435F_SR_WRITE_ENABLE_LATCH_BIT                                0x02
//! Write in progress bit
#define MX25R6435F_SR_WRITE_IN_PROGRESS_BIT                                 0x01

/*! Function for initializing MX25R6435F
 */
void MX25R6435F_Init( void );

/*! Function for erase whole chiip (CE) on MX25R6435F
 */
void MX25R6435F_Chip_Erase( void );

/*! Function for erase the selected block (BE) on MX25R6435F
* @param[in] address Address of the block to be erased
 */
void MX25R6435F_Block_Erase( uint32_t address );

/*! Function for erase the selected sector (SE) on MX25R6435F
* @param[in] address Address of the sector to be erased
 */
void MX25R6435F_Sector_Erase( uint32_t address );

/*! Function for program the selected page (PP) on MX25R6435F
* @param[in] address Address of the page to be programed
* @param[in] pData Pointer to the array of bytes to be programed
* @param[in] nData Number of bytes to be programed
 */
void MX25R6435F_Page_Program( uint32_t address, uint8_t *pData, uint32_t nData );

/*! Function for read n bytes (READ) on MX25R6435F
* @param[in] address Address of the page to be readed
* @param[in] pData Pointer to the array of bytes to be saved the data readed
* @param[in] nData Number of bytes to be readed
 */
void MX25R6435F_Page_Read( uint32_t address, uint8_t *pData, uint32_t nData );

/*! Function for enable power to MX25R6435F
 */
void MX25R6435F_Power_On( void );

/*! Function for disable power to MX25R6435F
 */
void MX25R6435F_Power_Off( void );

/*! Function for read the ID from IC MX25R6435F
 * @return The ID data of the device
 */
uint32_t MX25R6435F_Read_ID(void);

/*! Function for test 
* @return 1 when success, 0 when error
 */
uint8_t MX25R6435F_Test( void );

#endif /* __MX25R6435F_H_ */
